#include <SPI.h>
#include <Ethernet.h>
#include <SD.h>

// velkost bufferu pre požiadavky
#define REQ_BUF_SZ   20


byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 100, 203); // IP addresa
EthernetServer server(80);  // server na port 80
File webFile;
char HTTP_req[REQ_BUF_SZ] = {0}; // bufferovaná požiadavka 
char req_index = 0;              // index na HTTP_req buffer

void setup()
{
    // vypnutie Ethernet čipu
    pinMode(10, OUTPUT);
    digitalWrite(10, HIGH);
    
    Serial.begin(9600);       // zapnutie sériového portu pre ladenie
    
    // inicializácia SD karty
    Serial.println("Inicializacia SD karty...");
    if (!SD.begin(4)) {
        Serial.println("Chyba - SD karta nie je inicializovana");
        return;    
    }
    Serial.println("OK - SD karta inicializovana");
    // hľadáme súbor index.htm 
    if (!SD.exists("index.htm")) {
        Serial.println("Chyba - nemozem najst subor index.htm");
        return;  // návrat, ak nemôže nájsť súbor index.htm
    }
    Serial.println("OK - subor index.htm najdeny.");
    
    Ethernet.begin(mac, ip);  
    server.begin();           
}


void loop()
{
    EthernetClient client = server.available();  

    if (client) {  
        boolean prazdnyRiadok = true;
        while (client.connected()) {
            if (client.available()) {   // ak klient zaslal požiadavku
                char c = client.read(); // prečítaj 1 bajt od klienta 
                // do bufferu vloží HTTP požiadavku
                
                if (req_index < (REQ_BUF_SZ - 1)) {
                    HTTP_req[req_index] = c;          // ulož znak HTTP požiadavky
                    req_index++;
                }
                
                // posledný riadok klientskej požiadavky je prázdny riadok a končí znakom \n

                if (c == '\n' && prazdnyRiadok) {
                    
                    client.println("HTTP/1.1 200 OK");// zašle klasickú hlavičku
                    // táto časť hlavičky nižšie v závislosti na tom, či
                    // web stránka alebo XML stránka je požadovaná 
                    // Ajax požiadavka - zaslanie XML súboru
                    if (StrContains(HTTP_req, "ajax_inputs")) {
                        // zaslanie zvyšku hlavičky
                        client.println("Content-Type: text/xml");
                        client.println("Connection: keep-alive");
                        client.println();
                        // odoslanie XML súboru obsahujúceho vstupné stavy
                        XML_response(client);
                    }
                    else {  // odoslavie klasickej hlavičky
                        // send rest of HTTP header
                        client.println("Content-Type: text/html");
                        client.println("Connection: keep-alive");
                        client.println();
                        // send web page
                        webFile = SD.open("index.htm");        // otvor súbor index.htm
                        if (webFile) {
                            while(webFile.available()) {
                                client.write(webFile.read()); // zašli stránku klientovi
                            }
                            webFile.close();
                        }
                    }
                    // vypíš prijatú HTTP požiadavku na sériový port
                    Serial.print(HTTP_req);
                    // vymaž index bufferu a obsah bufferu nastav na nulu
                    req_index = 0;
                    StrClear(HTTP_req, REQ_BUF_SZ);
                    break;
                }
                // každý riadok textu prijatý od klienta končí znakmi \r\n 
                if (c == '\n') {
                    // posledný znak na riadku prijatého textu
                   
                    prazdnyRiadok = true;
                } 
                else if (c != '\r') {
                    
                    prazdnyRiadok = false;
                }
            } // koniec if (client.available())
        } // koniec if (client.available())
        delay(1);      // dajme čas klientovi prijať dáta
        client.stop(); // ukonči spojenie s klientom
    } // koniec if (client) 
}

// zaslanie XML súboru obsahujúceho analógové hodnoty
void XML_response(EthernetClient cl)
{
    int analog_val;
    
    cl.print("<?xml version = \"1.0\" ?>");
    cl.print("<inputs>");
    // čítaj analóg pin A2
    analog_val = analogRead(2);
    cl.print("<analog>");
    cl.print(analog_val);
    cl.print("</analog>");
    cl.print("</inputs>");
}

// nastavuje každý element reťazca na nulu (vymaže pole)
void StrClear(char *str, char length)
{
    for (int i = 0; i < length; i++) {
        str[i] = 0;
    }
}

// hľadá znaky sfind v reťazci str
// vráti hodnotu 1 ak znaky našiel
// vráti hodnotu 0 ak znaky nenašiel
char StrContains(char *str, char *sfind)
{
    char found = 0;
    char index = 0;
    char len;

    len = strlen(str);
    
    if (strlen(sfind) > len) {
        return 0;
    }
    while (index < len) {
        if (str[index] == sfind[found]) {
            found++;
            if (strlen(sfind) == found) {
                return 1;
            }
        }
        else {
            found = 0;
        }
        index++;
    }

    return 0;
}
